@@ -24,11 +24,13 @@ def test_create_an_account_with_initial_deposit_balances(target_client: RestClie
 @pytest.mark.parametrize(  # pyre-ignore
     "kyc_data",
     [
-        offchain.to_json(offchain.individual_kyc_data()),
-        offchain.to_json(offchain.entity_kyc_data()),
+        offchain.individual_kyc_data(),
+        offchain.entity_kyc_data(),
     ],
 )
-def test_create_an_account_with_minimum_valid_kyc_data(target_client: RestClient, kyc_data: str) -> None:
+def test_create_an_account_with_minimum_valid_kyc_data(
+    target_client: RestClient, kyc_data: offchain.KycDataObject
+) -> None:
     account = target_client.create_account(kyc_data=kyc_data)
     assert account.id
 
@@ -36,7 +38,7 @@ def test_create_an_account_with_minimum_valid_kyc_data(target_client: RestClient
 def test_create_an_account_with_valid_kyc_data_and_initial_deposit_balances(
     target_client: RestClient, currency: str
 ) -> None:
-    minimum_kyc_data = offchain.to_json(offchain.individual_kyc_data())
+    minimum_kyc_data = offchain.individual_kyc_data()
     account = target_client.create_account(kyc_data=minimum_kyc_data, balances={currency: 123})
     assert account.id
 
