@@ -125,13 +125,8 @@ def general_metadata(
 
     Give from_subaddress None for the case transferring from non-custodial to custodial account.
     Give to_subaddress None for the case transferring from custodial to non-custodial account.
-
-    Returns empty bytes array if from_subaddress and to_subaddress both are None.
     """
 
-    if from_subaddress is None and to_subaddress is None:
-        return b""
-
     metadata = diem_types.Metadata__GeneralMetadata(
         value=diem_types.GeneralMetadata__GeneralMetadataVersion0(
             value=diem_types.GeneralMetadataV0(  # pyre-ignore
